@@ -1,6 +1,7 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 2000, 2002, 2004, 2007-2009 Free Software Foundation, Inc.
+    Copyright (C) 2000, 2002, 2004, 2007-2010 Free Software Foundation,
+    Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -352,57 +353,6 @@ mac_free (PedDisk* disk)
 	free (mac_disk_data);
 }
 
-#ifndef DISCOVER_ONLY
-static int
-_clobber_part_map (PedDevice* dev)
-{
-        void *buf = ped_malloc (dev->sector_size);
-        if (!buf)
-                return 0;
-
-        int ok = 1;
-	PedSector sector;
-	for (sector=1; 1; sector++) {
-                if (!ped_device_read (dev, buf, sector, 1)) {
-                        ok = 0;
-                        break;
-                }
-		if (!_rawpart_check_signature (buf)) {
-                        ok = 1;
-                        break;
-                }
-		memset (buf, 0, dev->sector_size);
-		if (!ped_device_write (dev, buf, sector, 1)) {
-                        ok = 0;
-                        break;
-                }
-	}
-        free (buf);
-        return ok;
-}
-
-static int
-mac_clobber (PedDevice* dev)
-{
-	void *buf;
-	if (!ptt_read_sector (dev, 0, &buf))
-		return 0;
-
-	if (!_check_signature (buf)) {
-                free (buf);
-		return 0;
-        }
-
-        memset (buf, 0, dev->sector_size);
-        bool ok = ped_device_write (dev, buf, 0, 1);
-        free (buf);
-        if (!ok)
-		return 0;
-
-	return _clobber_part_map (dev);
-}
-#endif /* !DISCOVER_ONLY */
-
 static int
 _rawpart_cmp_type (const MacRawPartition* raw_part, const char* type)
 {
@@ -1095,7 +1045,9 @@ write_block_zero (PedDisk* disk, MacDiskData* mac_driverdata)
 	memcpy(&raw_disk->driverlist[0], &mac_driverdata->driverlist[0],
 			sizeof(raw_disk->driverlist));
 
-	return ped_device_write (dev, raw_disk, 0, 1);
+	int write_ok = ped_device_write (dev, raw_disk, 0, 1);
+        free (s0);
+	return write_ok;
 }
 
 static int
@@ -1161,7 +1113,9 @@ mac_write (PedDisk* disk)
 			       mac_disk_data->part_map_entry_count))
 		goto error_free_part_map;
 	free (part_map);
-	return write_block_zero (disk, mac_driverdata);
+	int write_ok = write_block_zero (disk, mac_driverdata);
+	free (mac_driverdata);
+	return write_ok;
 
 error_free_part_map:
 	free (part_map);
@@ -1425,6 +1379,14 @@ mac_partition_get_name (const PedPartition* part)
 	return mac_data->volume_name;
 }
 
+static PedAlignment*
+mac_get_partition_alignment(const PedDisk *disk)
+{
+        PedSector sector_size = disk->dev->sector_size / 512;
+
+        return ped_alignment_new(0, sector_size);
+}
+
 static PedConstraint*
 _primary_constraint (PedDisk* disk)
 {
@@ -1617,49 +1579,21 @@ mac_get_max_supported_partition_count (const PedDisk* disk, int *max_n)
 	return true;
 }
 
-static bool
-mac_partition_check (const PedPartition* part)
-{
-	return true;
-}
+#include "pt-common.h"
+PT_define_limit_functions (mac)
 
 static PedDiskOps mac_disk_ops = {
-	probe:			mac_probe,
-#ifndef DISCOVER_ONLY
-	clobber:		mac_clobber,
-#else
-	clobber:		NULL,
-#endif
-	alloc:			mac_alloc,
-	duplicate:		mac_duplicate,
-	free:			mac_free,
-	read:			mac_read,
-#ifndef DISCOVER_ONLY
+	clobber: NULL,
         /* FIXME: remove this cast, once mac_write is fixed not to
            modify its *DISK parameter.  */
-	write:			(int (*) (const PedDisk*)) mac_write,
-#else
-	write:			NULL,
-#endif
+	write:	NULL_IF_DISCOVER_ONLY ((int (*) (const PedDisk*)) mac_write),
 
-	partition_new:		mac_partition_new,
-	partition_duplicate:	mac_partition_duplicate,
-	partition_destroy:	mac_partition_destroy,
-	partition_set_system:	mac_partition_set_system,
-	partition_set_flag:	mac_partition_set_flag,
-	partition_get_flag:	mac_partition_get_flag,
-	partition_is_flag_available:	mac_partition_is_flag_available,
 	partition_set_name:	mac_partition_set_name,
 	partition_get_name:	mac_partition_get_name,
-	partition_align:	mac_partition_align,
-	partition_enumerate:	mac_partition_enumerate,
-	partition_check:	mac_partition_check,
 
-	alloc_metadata:		mac_alloc_metadata,
-	get_max_primary_partition_count:
-				mac_get_max_primary_partition_count,
-	get_max_supported_partition_count:
-				mac_get_max_supported_partition_count
+	get_partition_alignment: mac_get_partition_alignment,
+
+	PT_op_function_initializers (mac)
 };
 
 static PedDiskType mac_disk_type = {
