@@ -1,6 +1,6 @@
 #!/bin/sh
 
-# Copyright (C) 2008 Free Software Foundation, Inc.
+# Copyright (C) 2008-2010 Free Software Foundation, Inc.
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,6 +22,15 @@ privileges_required_=1
 
 : ${srcdir=.}
 . $srcdir/test-lib.sh
+require_xfs_
+ss=$sector_size_
+
+# On a 32-bit system, we must skip this test when $ss >= 4096.
+# Otherwise, due to an inherent 32-bit-XFS limit, dd would fail to
+# create the file of size > 16TiB
+if test $(uname -m) != x86_64; then
+  test $ss -le 2048 || exit 77
+fi
 
 ####################################################
 # Create and mount a file system capable of dealing with >=2TB files.
@@ -35,7 +44,7 @@ test_expect_success \
     'create an XFS file system' \
     '
     dd if=/dev/zero of=$fs bs=1MB count=2 seek=20 &&
-    mkfs.xfs -q $fs &&
+    mkfs.xfs -f -q $fs &&
     mkdir "$mp"
 
     '
@@ -57,7 +66,8 @@ do_mkpart()
   start_sector=$1
   end_sector=$2
   # echo '********' $(echo $end_sector - $start_sector + 1 |bc)
-  dd if=/dev/zero of=$dev bs=1b count=2k seek=$end_sector 2> /dev/null &&
+  dd if=/dev/zero of=$dev bs=$ss count=2k seek=$end_sector 2> dd-err ||
+    { cat dd-err 1>&2; return 1; }
   parted -s $dev mklabel $table_type &&
   parted -s $dev mkpart p xfs ${start_sector}s ${end_sector}s
 }
@@ -100,27 +110,13 @@ test_expect_success \
     'check for new diagnostic' \
     'bad_part_length 4294967296 > exp && diff -u err exp'
 
-# FIXME: investigate this.
-# Unexpectedly to me, both of these failed with this same diagnostic:
-#
-#   Error: partition length of 4294967296 sectors exceeds the \
-#   DOS-partition-table-imposed maximum of 2^32-1" > exp &&
-#
-# I expected the one below to fail with a length of _4294967297_.
-# Debugging, I see that _check_partition *does* detect this,
-# but the diagnostic doesn't get displayed because of the wonders
-# of parted's exception mechanism.
-
 test_expect_failure \
     "$table_type: a partition length of 2^32+1 sectors provokes failure." \
     'do_mkpart $n $(echo $n+2^32|bc) > err 2>&1'
 
-# FIXME: odd that we asked for 2^32+1, yet the diagnostic says 2^32
-# FIXME: Probably due to constraints.
-# FIXME: For now, just accept the current output.
 test_expect_success \
     'check for new diagnostic' \
-    'bad_part_length 4294967296 > exp && diff -u err exp'
+    'bad_part_length 4294967297 > exp && diff -u err exp'
 
 # =========================================================
 # Now consider partition starting sector numbers.
@@ -135,7 +131,7 @@ test_expect_success \
 cat > exp <<EOF
 Model:  (file)
 Disk: 4294970342s
-Sector size (logical/physical): 512B/512B
+Sector size (logical/physical): ${ss}B/${ss}B
 Partition Table: $table_type
 
 Number  Start        End          Size   Type     File system  Flags
@@ -162,7 +158,7 @@ test_expect_failure \
     'do_mkpart_start_and_len $(echo 2^32+1|bc) 1000 > err 2>&1'
 test_expect_success \
     'check for new diagnostic' \
-    'bad_start_sector 4294967296 > exp && diff -u err exp'
+    'bad_start_sector 4294967297 > exp && diff -u err exp'
 
 done
 
